==> nui_core-0.0.127.tar.gz: lib/config/nui_global_config.dart <==
import 'package:flutter/services.dart' show rootBundle;
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/config/nui_app_config.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/string/formatter/nui_printf.dart';

class NUIGlobalConfig {
  NUIGlobalConfig._builder(NUIGlobalConfigBuilder builder){
    _module = builder._module;
    _filename = builder._filename;
    _instance[_module] = this;
  }

  static String _MODULE = "NUIGlobalConfig";
  static Map<String, NUIGlobalConfig> _instance = Map();

  static NUIGlobalConfig get({String module}){
    final targetModule = module ?? _MODULE;
    final config = _instance[targetModule];
    if(config == null) throwNUI(_MODULE, "Instance has not been initialized with builder for module $targetModule");
    return config;
  }

  NUIAppConfig getConfig(){
    try {
      return NUIAppConfigEntity().toEntity(_appConfig);
    }catch(e){
      return null;
    }
  }

  Map<String, dynamic> _appConfig = Map<String, dynamic>();
  String _module;
  String _filename;

  NUIGlobalConfig loadFromMap(Map<String, dynamic> map) {
    _appConfig.addAll(map);
    return this;
  }

  Future<NUIGlobalConfig> loadFromAsset(String name) async {
    if (!name.endsWith(".json")) {
      name = "$name.json";
    }
    try {
      logNUI(_MODULE, "Loading configs from file : $name");
      String content = await rootBundle.loadString("assets/build/$name");
      if(isNullOrEmpty(content)){
        throwNUI(_MODULE, "Config file is empty for ${"assets/build/$name"}");
      }
      Map<String, dynamic> configAsMap = json.decode(content);
      logNUI(_MODULE, "Content of config from file : $content");
      _appConfig.addAll(configAsMap);
      return this;
    }catch(e){
      throwNUI(_MODULE, "No config file specified for $name with error : $e");
    }
  }

  Future<NUIGlobalConfig> loadFromPath(String path) async {
    String content = await rootBundle.loadString(path);
    Map<String, dynamic> configAsMap = json.decode(content);
    _appConfig.addAll(configAsMap);
    return this;
  }

  Future<NUIGlobalConfig> loadFromPathIntoKey(String path, String key) async {
    String content = await rootBundle.loadString(path);
    Map<String, dynamic> configAsMap = json.decode(content);
    _appConfig.putIfAbsent(key, () => configAsMap);
    return this;
  }

  Future<NUIGlobalConfig> loadFromUrl(String url, {Map<String, String> queryParameters, Map<String, String> headers}) async {
    Map<String, dynamic> configAsMap = await _getFromUrl(url, queryParameters: queryParameters, headers: headers);
    _appConfig.addAll(configAsMap);
    return this;
  }

  Future<NUIGlobalConfig> loadFromUrlIntoKey(String url, String key, {Map<String, String> queryParameters, Map<String, String> headers}) async {
    Map<String, dynamic> configAsMap = await _getFromUrl(url, queryParameters: queryParameters, headers: headers);
    _appConfig.putIfAbsent(key, () => configAsMap);
    return this;
  }

  Map<String, dynamic> getAll(){
    return _appConfig;
  }

  dynamic getDynamic(String key) => _appConfig[key];

  T getValue<T>(String key) => _appConfig[key] as T;

  String getString(String key, {List<dynamic> params}){
    final value = getValue<String>(key);
    if(isNullOrEmpty(value)) return null;

    return printNUI(value, params);
  }

  T getDeepValue<T>(String keyPath) {
    dynamic _value;
    keyPath.split(":").forEach((element) {
      if (_value == null)
        _value = _appConfig[element];
      else
        _value = _value[element];
    });
    return null;
  }

  void clear() => _appConfig.clear();

  void updateValue(String key, dynamic value) {
    if (_appConfig[key] != null && value.runtimeType != _appConfig[key].runtimeType) {
      throwNUI(_MODULE, "The type of ${_appConfig[key].runtimeType} does not match the given type ${value.runtimeType}");
    }
    _appConfig.update(key, (dynamic) => value);
  }

  void addValue(String key, dynamic value) => _appConfig.putIfAbsent(key, () => value);

  void addValues(Map<String, dynamic> map) => _appConfig.addAll(map);

  Future<Map<String, dynamic>> _getFromUrl(String url, {Map<String, String> queryParameters, Map<String, String> headers}) async {
    String finalUrl = url;
    if (queryParameters != null) {
      queryParameters.forEach((k, v) {
        finalUrl += !finalUrl.endsWith("?") ? "?$k=$v" : "&$k=$v";
      });
    }
    if (headers == null) {
      headers = Map<String, String>();
    }
    headers.putIfAbsent("Accept", () => "application/json");
    var response = await http.get(Uri.encodeFull(finalUrl), headers: headers);
    if (response == null || response.statusCode != 200) {
      throw new Exception('HTTP request failed, statusCode: ${response?.statusCode}, $finalUrl');
    }
    return json.decode(response.body);
  }
}

class NUIGlobalConfigBuilder{
  String _module;
  String _filename;

  NUIGlobalConfigBuilder module(String module){
    this._module = module;
    return this;
  }

  NUIGlobalConfigBuilder fileName(String filename){
    this._filename = filename;
    return this;
  }

  Future<NUIGlobalConfig> build() async{
    final config = NUIGlobalConfig._builder(this);
    if(!isNullOrEmpty(config._filename)){
      await config.loadFromAsset(config._filename);
    }
    return config;
  }
}

==> nui_core-0.0.250.tar.gz: lib/config/nui_global_config.dart <==
import 'package:flutter/services.dart' show rootBundle;
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/config/nui_app_config.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/string/formatter/nui_printf.dart';

class NUIGlobalConfig {
  NUIGlobalConfig._builder(NUIGlobalConfigBuilder builder){
    _module = builder._module;
    _filename = builder._filename;
    _instance[_module] = this;
  }

  static String _MODULE = "NUIGlobalConfig";
  static Map<String, NUIGlobalConfig> _instance = Map();

  static NUIGlobalConfig get({String module}){
    final targetModule = module ?? _MODULE;
    final config = _instance[targetModule];
    if(config == null) throwNUI(_MODULE, "Instance has not been initialized with builder for module $targetModule");
    return config;
  }

  NUIAppConfig getConfig(){
    try {
      return NUIAppConfigEntity().toEntity(_appConfig);
    }catch(e){
      return null;
    }
  }

  Map<String, dynamic> _appConfig = Map<String, dynamic>();
  String _module;
  String _filename;

  NUIGlobalConfig loadFromMap(Map<String, dynamic> map) {
    _appConfig.addAll(map);
    return this;
  }

  Future<NUIGlobalConfig> loadFromAsset(String name) async {
    if (!name.endsWith(".json")) {
      name = "$name.json";
    }
    try {
      logNUI(_MODULE, "Loading configs from file : $name");
      String content = await rootBundle.loadString("assets/build/$name");
      if(isNullOrEmpty(content)){
        throwNUI(_MODULE, "Config file is empty for ${"assets/build/$name"}");
      }
      Map<String, dynamic> configAsMap = json.decode(content);
      logNUI(_MODULE, "Content of config from file : $content");
      _appConfig.addAll(configAsMap);
      return this;
    }catch(e){
      throwNUI(_MODULE, "No config file specified for $name with error : $e");
    }
  }

  Future<NUIGlobalConfig> loadFromPath(String path) async {
    String content = await rootBundle.loadString(path);
    Map<String, dynamic> configAsMap = json.decode(content);
    _appConfig.addAll(configAsMap);
    return this;
  }

  Future<NUIGlobalConfig> loadFromPathIntoKey(String path, String key) async {
    String content = await rootBundle.loadString(path);
    Map<String, dynamic> configAsMap = json.decode(content);
    _appConfig.putIfAbsent(key, () => configAsMap);
    return this;
  }

  Future<NUIGlobalConfig> loadFromUrl(String url, {Map<String, String> queryParameters, Map<String, String> headers}) async {
    Map<String, dynamic> configAsMap = await _getFromUrl(url, queryParameters: queryParameters, headers: headers);
    _appConfig.addAll(configAsMap);
    return this;
  }

  Future<NUIGlobalConfig> loadFromUrlIntoKey(String url, String key, {Map<String, String> queryParameters, Map<String, String> headers}) async {
    Map<String, dynamic> configAsMap = await _getFromUrl(url, queryParameters: queryParameters, headers: headers);
    _appConfig.putIfAbsent(key, () => configAsMap);
    return this;
  }

  Map<String, dynamic> getAll(){
    return _appConfig;
  }

  dynamic getDynamic(String key) => _appConfig[key];

  T getValue<T>(String key) => _appConfig[key] as T;

  String getString(String key, {List<dynamic> params}){
    final value = getValue<String>(key);
    if(isNullOrEmpty(value)) return null;

    return printNUI(value, params);
  }

  T getDeepValue<T>(String keyPath) {
    dynamic _value;
    keyPath.split(":").forEach((element) {
      if (_value == null)
        _value = _appConfig[element];
      else
        _value = _value[element];
    });
    return null;
  }

  void clear() => _appConfig.clear();

  void updateValue(String key, dynamic value) {
    if (_appConfig[key] != null && value.runtimeType != _appConfig[key].runtimeType) {
      throwNUI(_MODULE, "The type of ${_appConfig[key].runtimeType} does not match the given type ${value.runtimeType}");
    }
    _appConfig.update(key, (dynamic) => value);
  }

  void addValue(String key, dynamic value) => _appConfig.putIfAbsent(key, () => value);

  void addValues(Map<String, dynamic> map) => _appConfig.addAll(map);

  Future<Map<String, dynamic>> _getFromUrl(String url, {Map<String, String> queryParameters, Map<String, String> headers}) async {
    String finalUrl = url;
    if (queryParameters != null) {
      queryParameters.forEach((k, v) {
        finalUrl += !finalUrl.endsWith("?") ? "?$k=$v" : "&$k=$v";
      });
    }
    if (headers == null) {
      headers = Map<String, String>();
    }
    headers.putIfAbsent("Accept", () => "application/json");
    var response = await http.get(Uri.parse(finalUrl), headers: headers);
    if (response == null || response.statusCode != 200) {
      throw new Exception('HTTP request failed, statusCode: ${response?.statusCode}, $finalUrl');
    }
    return json.decode(response.body);
  }
}

class NUIGlobalConfigBuilder{
  String _module;
  String _filename;

  NUIGlobalConfigBuilder module(String module){
    this._module = module;
    return this;
  }

  NUIGlobalConfigBuilder fileName(String filename){
    this._filename = filename;
    return this;
  }

  Future<NUIGlobalConfig> build() async{
    final config = NUIGlobalConfig._builder(this);
    if(!isNullOrEmpty(config._filename)){
      await config.loadFromAsset(config._filename);
    }
    return config;
  }
}
